from django.shortcuts import render
from django.shortcuts import render,redirect
from django.contrib.auth.decorators import user_passes_test
from django.utils.decorators import method_decorator
from rest_framework.response import Response
from rest_framework import status
from rest_framework.views import APIView
# Create your views here.
from clmainApp.utils import getClinicId
from rcpmainApp.utils import getBranchId
from brpatientApp.models import Patient
from .serializers import *
from brvisitApp.models import PatientVisit
from brmainApp.models import Location
from clmainApp.models import FoundWay

from brsessionApp.models import PatientSession
class BranchPatientsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('Reception.add_patient'), login_url='/auth/login/'))    
    def post(self,request):
        #import pdb
        #pdb.set_trace()
        clinicId = getClinicId(request)
        if not clinicId : return Response(status=status.HTTP_400_BAD_REQUEST)
        request.data._mutable = True
        patients = Patient.objects.filter(clinic_id=clinicId).aggregate(Max('code'))
        new_code_value = int(patients["code__max"])+1 if patients["code__max"] else 1
        request.data.update({"created_by":request.user.id,"clinic":clinicId,"code":new_code_value})
        serializedObj= PatientSerializer(data=request.data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data , status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class PatientApi(APIView):
    def get_object(self,pk):
        try:
            return Patient.objects.get(pk=pk)
        except Patient.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('Reception.view_patient'), login_url='/auth/login/'))        
    def get(self,request,pk): 
        patientObj=self.get_object(pk)
        if patientObj :
            patient_serialized=PatientSerializer(patientObj)
            context = {"patient":patient_serialized.data,'activePage':'Patients'}
            # patientVisitObjs = PatientVisit.objects.filter(patient=patientObj,visit_status__in=["1I","4F"]).order_by('-visit_day')
            # patientVisitObjs_serialized = PatientVisitMiniSerializer(patientVisitObjs,many=True)
            # context.update({'patientVisitObjs':patientVisitObjs_serialized.data})
            # try : 
            #     patientInfoObj = PatientInfo.objects.get(patient=patientObj)
            #     context.update({'pI':patientInfoObj})
            # except PatientInfo.DoesNotExist:
            #     context.update({'pI':None})
            
            #context.update({"marital_status":MARITAL_STATUS_CHOICES,})
            #context.update({"blood_types":BLOOD_TYPE_CHOICES,})
            #context.update({"child_sex":CHILD_SEX_CHOICES,})
            # gender_choices = ChoiceField(choices=GENDER_CHOICES)
            # context.update({"genders":gender_choices.choices,})
            # childbirth_method = ChoiceField(choices=CHILDBIRTH_METHODS_CHOICES)
            # context.update({"childbirth_methods":childbirth_method.choices,})
            #context.update({"pageTitle":patientObj.fullname,})
            return render(request,"patientApp/Reception/patient.html",context)

        #     serializeobj= PatientSerializer(obj)
        #     return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return redirect("brpatientApp:PatientsApi")

    #@method_decorator(user_passes_test(lambda u: u.has_perm('Reception.change_patient'), login_url='/auth/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        # clinicId = getClinicId(request)
        # if not clinicId : return redirect('clauthApp:home')
        request.data._mutable = True
        #request.data.update({"created_by":request.user.id,"clinic":clinicId})
        serializedObj= PatientSerializer(obj,data=request.data,partial=True)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: u.has_perm('Reception.delete_patient'), login_url='/auth/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

from django.core.serializers.json import DjangoJSONEncoder
import json
from clvoucherApp.serializers import PatientVoucherSerializer,PatientVoucher
class PatientInquireApi(APIView): 
    def get(self,request,patient_pk):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        patient =Patient.objects.get(pk=patient_pk)
        context = {"patient":patient,'activePage':'PatientInfo','pageTitle':patient.fullname}
        #patient_age = ageCalculator(patient.birth_date)
        p_visits = PatientVisit.objects.filter(patient=patient_pk).order_by('-created_at')
        context.update({'p_visits':p_visits})
        #p_sessions = PatientSession.objects.filter(patient=patient_pk).order_by('-created_at')
        context.update({'p_visits':p_visits,
                        #"p_sessions":p_sessions
                        })
        context.update({"locations":Location.objects.filter(branch_id=branchId),"found_ways":FoundWay.objects.all(),})
        #pt_vouchers =PatientVoucher.objects.filter(patient_id=patient_pk).order_by("-is_used_session","-created_at")
        #pt_vouchers_srl=PatientVoucherSerializer(pt_vouchers,many=True)
        # Convert to JSON string using Django's encoder
        #pt_vouchers_json = json.dumps(pt_vouchers_srl.data, cls=DjangoJSONEncoder)
        #context.update({'pt_vouchers_json':pt_vouchers_json})
        return render(request,"Reception/rcppatientApp/patient_info.html",context)
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('Reception.change_patient'), login_url='/auth/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk=pk)
        request.data._mutable = True
        request.data.update({"created_by":request.user.id})
        # request.data.update({"organization": None if request.data["organization"] == "0" else request.data["organization"]})
        # request.data.update({"company": None if request.data["company"] == "0" else request.data["company"]})
        serializedObj= PatientSerializer(obj,data=request.data,partial=True)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)